let Web3 = require('web3')
const Web3EthContract = require('web3-eth-contract')
let web3

async function init(provider) {
  web3 = new Web3(new Web3.providers.HttpProvider(provider))
  return
}

async function getBlock(blocknumber) {
  return new Promise(async (resolve, reject) => {
    let block = await web3.eth.getBlock(blocknumber)
    resolve(block)
  })
}

async function getBalance(address) {
  return new Promise(async (resolve, reject) => {
    let balance = await web3.eth.getBalance(address)
    resolve(balance)
  })
}

/**
 * Converts any wei value into an ether value.
 * @param {string} value
 * @param {string} unit 'ether'
 * @returns string number
 */
function formatBalance(value, unit) {
  return new Promise(async (resolve, reject) => {
    let formatedBalance = await web3.utils.fromWei(value, unit)
    resolve(formatedBalance)
  })
}

/**
 * Signs an Ethereum transaction with a given private key.
 * @param {object} tx
 * @param {string} privateKey
 * @returns {object}
 */
function signTransaction(tx, privateKey) {
  return new Promise(async (resolve, reject) => {
    let signedTransaction = await web3.eth.accounts.signTransaction(
      tx,
      privateKey
    )
    resolve(signedTransaction)
  })
}

/**
 * Sends an already signed transaction
 * @param {object} signedTransactionData
 * @returns Transaction hash
 */
async function sendTransaction(signedTransactionData) {
  return new Promise(async (resolve, reject) => {
    let receipt = await web3.eth.sendSignedTransaction(signedTransactionData)
    resolve(receipt)
  })
}

async function initContract(provider, abi, address) {
  return new Promise((resolve, reject) => {
    Web3EthContract.setProvider(provider)
    const contract = new Web3EthContract(abi, address)
    resolve(contract)
  })
}

module.exports = {
  init,
  getBlock,
  getBalance,
  formatBalance,
  signTransaction,
  sendTransaction,
  initContract,
}
